package com.tptj.tool.hg.fineui.swing.element;

import com.eclipsesource.v8.V8Array;
import com.eclipsesource.v8.V8Object;
import com.fr.general.GeneralUtils;
import com.fr.log.FineLoggerFactory;
import com.tptj.tool.hg.fineui.swing.FineUIEngine;
import com.tptj.tool.hg.fineui.swing.element.layout.LayoutBuilder;
import com.tptj.tool.hg.fineui.swing.element.layout.LayoutType;
import com.tptj.tool.hg.fineui.swing.stable.ColorUtils;
import org.nfunk.jep.function.Abs;

import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.util.List;

/**
 * @author 秃破天际
 * @version 10.0
 * Created by 秃破天际 on 2021/10/29
 **/
public abstract class AbstractElement<T extends JComponent> implements Element<T> {

    protected T component;
    protected JComponent wrapper;
    private V8Object ref;
    private String type;
    //记录当前的控件状态，用于驱动一些特殊事件
    private Status status;

    public AbstractElement(T component) {
        this.wrapper = component;
        this.component = component;
        component.setOpaque(false);
    }

    public void setComponent(T component) {
        this.wrapper = component;
        this.component = component;
        component.setOpaque(false);
    }

    private Map<String, Object> attributes = new HashMap<String, Object>();
    private Map<String, Object> data = new HashMap<String, Object>();
    private Set<String> clazz_set = new HashSet<String>();
    private Map<String, Object> css = new HashMap<String, Object>();
    private Object val;


    private AbstractElement parent;
    private List<AbstractElement> children = new ArrayList<AbstractElement>();

    public AbstractElement() {
    }

    public Status getStatus() {
        return status;
    }

    public void setStatus(Status status) {
        this.status = status;
    }

    @Override
    public JComponent getWrapper() {
        return wrapper;
    }

    @Override
    public T getComponent() {
        return component;
    }

    @Override
    public void setType(String type) {
        this.type = type;
    }

    @Override
    public void mounted() {

    }

    @Override
    public AbstractElement parent() {
        return parent;
    }

    @Override
    public void setParent(AbstractElement parent) {
        this.parent = parent;
    }

    @Override
    public void ref(V8Object obj) {
        this.ref = obj;
    }

    @Override
    public void removeClass(String clazz) {
        clazz_set.remove(clazz);
    }

    @Override
    public void setData(String name, Object data) {
        this.data.put(name, data);
    }

    @Override
    public void css(String name, String val) {
        switch (name) {
            case "overflow":
            case "overflowX":
            case "overflowY":
            case "overflow-x":
            case "overflow-y":
                if ("auto".equals(val)) {
                    wrapper = new JScrollPane(component);
                }
                break;
            case "background":
                component.setOpaque(true);
                Color color = ColorUtils.str2Color(val);
                component.setBackground(color);
                break;
            case "color":
                color = ColorUtils.str2Color(val);
                component.setForeground(color);
                break;
            default:
                break;
        }
        css.put(name, val);
    }

    @Override
    public Map<String, Object> getStyle() {
        return css;
    }

    @Override
    public void cssMap(V8Object map) {
        String[] keys = map.getKeys();
        for (String key : keys) {
            css(key, GeneralUtils.objectToString(map.get(key)));
        }
    }

    @Override
    public void unbind() {

    }

    @Override
    public void updateUI() {
        wrapper.updateUI();
    }

    @Override
    public void destroy() {
//        wrapper.getParent().remove(wrapper);
//        children.clear();
    }

    @Override
    public boolean hasScrollbar() {
        return wrapper != component;
    }

    @Override
    public Object getData(String name) {
        return data.get(name);
    }

    @Override
    public void addClass(String clazz) {
        clazz_set.add(clazz);
    }

    @Override
    public void setAttr(String name, Object val) {
        attributes.put(name, val);
    }

    @Override
    public Object getAttr(String name) {
        return attributes.get(name);
    }

    @Override
    public void empty() {
        children.clear();
        component.removeAll();
        component.setLayout(null);
    }

    @Override
    public void layout(String layoutType) {
        LayoutType layout = LayoutBuilder.layout(layoutType, this);
        component.setLayout(layout.init());
        for (int i = 0, len = children.size(); i < len; i++) {
            AbstractElement childElement = children.get(i);
            layout.layout(childElement, i);
        }
    }

    @Override
    public void append(AbstractElement element) {
        if (null == element.getComponent()) {
            List<AbstractElement> children = element.children();
            for (AbstractElement child : children) {
                append(child);
            }
        } else {
            children.add(element);
            element.setParent(this);
        }
    }

    @Override
    public List<AbstractElement> children() {
        return children;
    }

    @Override
    public void appendTo(AbstractElement element) {
        parent = element;
        element.append(this);
    }

    @Override
    public void appendChild(AbstractElement element) {
        append(element);
    }

    @Override
    public void setWidth(String width) {
        css("width", width);
    }

    @Override
    public void setHeight(String height) {
        css("height", height);
    }

    @Override
    public int getWidth() {
        return component.getWidth();
    }

    @Override
    public int getHeight() {
        return component.getHeight();
    }

    @Override
    public void setVal(Object val) {
        this.val = val;
    }

    protected Object callJsMethod(String name, Object... args) {
        try {
            return FineUIEngine.call(ref, name, args);
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e, e.getMessage());
        }
        return null;
    }

    @Override
    public Object getVal() {
        return val;
    }

    @Override
    public void populate(V8Array items) {

    }

    @Override
    public int getRectStyleAttr(String name, int max, int min) {
        Object val = css.get(name);
        if ("".equals(val)) {
            return min;
        }
        if (null == val) {
            return min;
        }
        if (val instanceof String && ((String) val).endsWith("%")) {
            String sv = (String) val;
            sv = sv.substring(0, sv.length() - 1);
            double vald = GeneralUtils.objectToNumber(sv).doubleValue() * max;
            vald = vald / 100;
            return (int) vald;
        }
        return (null == val || "initial".equals(val) || min > GeneralUtils.objectToNumber(val).intValue())
                ? min : GeneralUtils.objectToNumber(val).intValue();
    }
}
