package com.tptj.tool.hg.fineui.swing.element;

import com.fr.design.gui.ipasswordfield.UIPassWordField;
import com.fr.design.gui.itextfield.UITextField;
import com.fr.general.GeneralUtils;

import com.fr.stable.StringUtils;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

/**
 * @author 秃破天际
 * @version 10.0
 * Created by 秃破天际 on 2021/11/2
 **/
public class TextElement extends AbstractSingleElement<JTextField>{
    public TextElement() {
        super(new UITextField());
        init();
    }
    private void validationChecker(){
        callJsMethod("validationChecker");
    }
    private boolean quitChecker(){
        return (boolean) callJsMethod("quitChecker");
    }
    private void init() {
        JTextField wgt =  getComponent();
        wgt.getDocument().addDocumentListener(new DocumentListener() {
            @Override
            public void insertUpdate(DocumentEvent e) {
                validationChecker();
                callJsMethod("fireEvent","EVENT_CHANGE", wgt.getText() );
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                validationChecker();
                callJsMethod("fireEvent","EVENT_CHANGE", wgt.getText() );
                if( StringUtils.isEmpty( wgt.getText() ) ){
                    callJsMethod("fireEvent","EVENT_EMPTY");
                }
            }

            @Override
            public void changedUpdate(DocumentEvent e) {
                validationChecker();
                callJsMethod("fireEvent","EVENT_CHANGE", wgt.getText() );
                if( StringUtils.isEmpty( wgt.getText() ) ){
                    callJsMethod("fireEvent","EVENT_EMPTY");
                }
            }
        });
        wgt.addFocusListener(new FocusListener() {
            @Override
            public void focusGained(FocusEvent e) {
                setStatus(Status.FOCUS);
                callJsMethod("fireEvent","EVENT_FOCUS" );
            }

            @Override
            public void focusLost(FocusEvent e) {
                setStatus(Status.FREE);
                callJsMethod("fireEvent","EVENT_BLUR" );
                if( quitChecker() ){
                    callJsMethod("fireEvent","EVENT_CONFIRM" );
                }
            }
        });
        /**
        EVENT_CHANGE	输入值改变事件
        EVENT_FOCUS	focus事件
        EVENT_BLUR	blur事件
        EVENT_KEY_DOWN	keydown事件，300毫秒debounce才会触发
        EVENT_ENTER	回车按键事件，当quitChecker为null或检查值不等于false时触发
        EVENT_SPACE	空格按键事件
        EVENT_BACKSPACE	删除按键事件，backspace或del按键触发

        EVENT_START	开始输入事件
        EVENT_PAUSE	暂停输入事件
        EVENT_STOP	停止输入事件
        EVENT_CONFIRM	退出输入状态且当前值为有效值时表示的确定事件
        EVENT_CHANGE_CONFIRM	当confirm事件触发时如果当前值与上一次有效值不同就会触发该事件
        EVENT_REMOVE	清除事件，当EVENT_BACKSPACE触发时，删除到空的时候触发
        EVENT_EMPTY	空事件，输入框无值时触发
        EVENT_VALID	有效事件，输入框值有效触发
        EVENT_ERROR	无效事件，输入框值无效触发
        EVENT_RESTRICT	被限制事件，当回车键按下但是quitChecker检查为false，表示不允许被退出
         **/
        wgt.addKeyListener(new KeyListener() {
            @Override
            public void keyTyped(KeyEvent e) {

            }
            @Override
            public void keyPressed(KeyEvent e) {
                char c = e.getKeyChar();
                if( KeyEvent.VK_SPACE == (int)c ){
                    callJsMethod("fireEvent","EVENT_SPACE" );
                }else if( KeyEvent.VK_BACK_SPACE == (int)c  || KeyEvent.VK_DELETE == (int)c ){
                    callJsMethod("fireEvent","EVENT_BACKSPACE" );
                }else if( KeyEvent.VK_ENTER == (int)c ){
                    callJsMethod("fireEvent","EVENT_BACKSPACE" );
                    if( StringUtils.isEmpty( wgt.getText() ) ){
                        callJsMethod("fireEvent","EVENT_REMOVE" );
                    }
                }
                if( Status.FOCUS == getStatus() ){
                    callJsMethod("fireEvent","EVENT_START" );
                }
                setStatus(Status.EDITING);
                callJsMethod("fireEvent","EVENT_KEY_DOWN" );
            }
            @Override
            public void keyReleased(KeyEvent e) {

            }
        });
    }

    @Override
    public Object getVal(){
        return getComponent().getText();
    }

    @Override
    public void setVal( Object val ){
        super.setVal(val);
        getComponent().setText( GeneralUtils.objectToString(val) );
    }

    @Override
    public void css(String name, String val){
        super.css(name, val);
    }

    @Override
    public void setAttr(String name, Object val){
        super.setAttr(name, val);
        name = name.toLowerCase();
        if("type".equals(name) && "password".equals(val)){
            //换JAVA控件只能在这里换，之后就不允许换了！否则会导致渲染的样式丢失
            setComponent( new UIPassWordField() );
            init();
        }
    }
}
